require 'locomotive_plugins'
require 'json'
require 'uri'

require_relative 'booking_document'
require_relative 'helper'

module Locomotive
    module Booking
        # Tag that changes existing bookings or creates them anew if they don't
        # exist yet.
        class PutBookingsTag < ::Liquid::Tag
            BOOKING_RESOURCE_ID_KEY = "resourceId"
            BOOKING_DATA_KEY        = "booking_data"
            BOOKING_YEAR_KEY        = "year"
            BOOKING_WEEK_KEY        = "week"
            BOOKING_WEEKDATA_KEY    = "weekData"

            def initialize(tag_name, markup, tokens, context)
                @booking_data = Locomotive::Booking::Helper.markupScanForKey(BOOKING_DATA_KEY, markup)
                super
            end

            # The render method returns a JSON object that denotes either
            # success or failure of saving the given booking data.
            def render(context)
                booking_data = Locomotive::Booking::Helper.evaluateWithValueAndContext(@booking_data, context)
                ::Locomotive.log("PutBookingsTag: booking_data: #{@booking_data}")

                ## Decode the booking data from the query string.
                booking_data_hash = Locomotive::Booking::Helper.unescapeAndParseURIToJSON(booking_data)
                if booking_data_hash.nil?
                    ::Locomotive.log("PutBookingsTag: No booking_data_hash")
                    return Locomotive::Booking::Helper.failure_json
                end

                ## Retrieve the values and make some sanity checks.
                resource_id = booking_data_hash[BOOKING_RESOURCE_ID_KEY]

                year = booking_data_hash[BOOKING_YEAR_KEY]
                week = booking_data_hash[BOOKING_WEEK_KEY]
                weekData = booking_data_hash[BOOKING_WEEKDATA_KEY]

                if resource_id.nil? or year.nil? or week.nil? or weekData.nil?
                    ::Locomotive.log("PutBookingsTag: resourceId, year, week or weekdata nil")
                    return Locomotive::Booking::Helper.failure_json
                end

                ## Check if there is already an entry for the given resource_id, year and week.
                found_booking_data = BookingDocument::Availability.where(resource_id: resource_id, year: year, week: week).first
                if not found_booking_data
                    ## Need to create a new entry.
                    found_booking_data = BookingDocument::Availability.new
                    found_booking_data.resource     = resource_id
                    found_booking_data.year         = year
                    found_booking_data.week         = week
                end

                ## Regardless of whether the booking data already exists or
                ## not, the weekData must always be updated.
                found_booking_data.weekData = weekData
                found_booking_data.save

                ::Locomotive.log("PutBookingsTag: successfully wrote data with week: #{week} and year: #{year} and resource_id: #{resource_id}")
                Locomotive::Booking::Helper.success_json
            end

            def render_disabled(context)
                nil.to_json
            end
        end
    end
end
