require 'locomotive_plugins'
require 'json'
require 'uri'

require_relative 'booking_document'
require_relative 'helper'

module Locomotive
    module Booking
        # Tag that changes existing resources or creates them anew if they
        # don't exist yet.
        class PutResourcesTag < ::Liquid::Tag
            RESOURCES_DATA_KEY  = "resource_data"
            RESOURCE_ID_KEY     = "resourceId"
            RESOURCE_NAME_KEY   = "resourceName"
            RESOURCE_DELETE_KEY = "delete"

            def initialize(tag_name, markup, tokens, context)
                @resource_data = Locomotive::Booking::Helper.markupScanForKey(RESOURCES_DATA_KEY, markup)
                super
            end

            # The render method returns a JSON object that denotes either
            # success or failure of saving the given resource data.
            def render(context)
                ## Retrieve the resource hash from the query sring.
                resource_data = Locomotive::Booking::Helper.evaluateWithValueAndContext(@resource_data, context)
                resource_data_hash = Locomotive::Booking::Helper.unescapeAndParseURIToJSON(resource_data)

                if resource_data_hash.nil?
                    ::Locomotive.log("PutResourcesTag: empty resource_data_hash")
                    return Locomotive::Booking::Helper.failure_json
                end

                ## Retrieve the values from the resource
                resource_id     = resource_data_hash[RESOURCE_ID_KEY]
                resource_name   = resource_data_hash[RESOURCE_NAME_KEY]
                delete          = resource_data_hash[RESOURCE_DELETE_KEY]

                if resource_id and delete
                    deleteResource(resource_id)
                elsif resource_id and resource_name
                    changeResource(resource_id, resource_name)
                elsif resource_id.nil? and resource_name
                    newResource(resource_name)
                else
                    ::Locomotive.log("PutResourcesTag: Can't handle resource_id: #{resource_id} and resource_name: #{resource_name}")
                    Locomotive::Booking::Helper.failure_json
                end
            end

            def render_disabled(context)
                nil.to_json
            end

            private

            # Changes an existing resource with the given name.
            def changeResource(resource_id, resource_name)
                found_resource = searchResourceById(resource_id)

                if not found_resource
                    ::Locomotive.log("PutResourcesTag: Can't change resource_id: #{resource_id} as it couldn't be found.")
                    Locomotive::Booking::Helper.failure_json
                else
                    found_resource.name = resource_name
                    found_resource.save

                    Locomotive::Booking::Helper.success_json
                end
            end

            # Creates a new resource with the given name.
            def newResource(resource_name)
                new_resource = BookingDocument::Resource.new
                new_resource.name = resource_name
                new_resource.save

                Locomotive::Booking::Helper.success_json
            end

            # Deletes the resource of the given id.
            def deleteResource(resource_id)
                found_resource = searchResourceById(resource_id)

                if not found_resource
                    Locomotive::Booking::Helper.failure_json
                else
                    found_resource.delete
                    found_resource.save

                    Locomotive::Booking::Helper.success_json
                end
            end

            # Searches a resource document by id.
            #
            # This either returns the fouond document or nil if it couldn't be
            # found.
            def searchResourceById(resource_id)
                begin
                    BookingDocument::Resource.find(resource_id)
                rescue ::Mongoid::Errors::DocumentNotFound
                    ::Locomotive.log("PutResourcesTag: Couldn't find document for resource_id: #{resource_id}")
                    nil
                end
            end
        end
    end
end
